# R codes used to generate gene set analysis results for evaluation of the permutation methods with human primary cell data
## Pashupati Mishra
## Liisa Holm's Bioinformatics lab
## Institute of Biotechnology, University of Helsinki
## 28.06.2016


load("expr.data.atlas.RData")
load("gene.sets.atlas.RData")
source("mGSZm.wrapper.R") 

# gene set data in matrix form
gene.sets.atlas.mat <- toMatrix(expr.data.atlas,gene.sets.atlas,flip.gene.sets=FALSE)

# gene set data in camera format
gene.sets.atlas.camera <- list()
for(i in 1:ncol(gene.sets.atlas.mat)){
    gene.sets.atlas.camera[[i]] <- as.numeric(which(gene.sets.atlas.mat[,i]==1))
}
names(gene.sets.atlas.camera) <- colnames(gene.sets.atlas.mat)

# gene set data in qusage format
gene.sets.atlas.qusage <- list()
for(i in 1:ncol(gene.sets.atlas.mat)){
    gene.sets.atlas.qusage[[i]] <- rownames(expr.data.atlas)[as.numeric(which(gene.sets.atlas.mat[,i]==1))]
}

names(gene.sets.atlas.qusage) <- colnames(gene.sets.atlas.mat)


# gene set data in romer format
gene.sets.atlas.romer <- list()
for(i in 1:ncol(gene.sets.atlas.mat)){
    ind <- which(gene.sets.atlas.mat[,i]==1)
    gene.sets.atlas.romer[[i]] <- ind
}
names(gene.sets.atlas.romer) <- colnames(gene.sets.atlas.mat)


library(limma)
library(parallel)
library(doParallel)
library(foreach)
library(doMC)

n.cores <- detectCores()



cl <- makeCluster(1) # 50 thiyo
registerDoParallel(cl)

# extracting column names
col_names <- unlist(lapply(colnames(expr.data.atlas), function(x) strsplit(x,":")[[1]][2]))
table(col_names)

uniq_col_names <- unique(col_names)
no.grps <- length(uniq_col_names)
no.replicates <- rep(NA,no.grps)

for(i in 1:no.grps){
  no.replicates[i] <- length(grep(uniq_col_names[i],col_names))
}
# Generating random samples for 100 splits data
set.seed(123)
iter <- 1 ## 100 thiyo
rand.ind <- vector("list",iter)
for(i in 1:iter){
  res <- matrix(NA,nrow=5,ncol=8)
  res[,1] <- sample(1:17,5,replace=F)
  res[,2] <- sample(18:40,5,replace=F)
  res[,3] <- sample(41:50,5,replace=F)
  res[,4] <- sample(51:60,5,replace=F)
  res[,5] <- sample(61:78,5,replace=F)
  res[,6] <- sample(79:90,5,replace=F)
  res[,7] <- sample(91:110,5,replace=F)
  res[,8] <- sample(111:124,5,replace=F)
  rand.ind[[i]] <- res
}



# sample labels
lbs_test <- unlist(lapply(c(1:8),function(x) rep(x,5)))
lbs_ref <- rep(1:8,(no.replicates-5))
lbs_camera_test <- unlist(lapply(letters[1:8],function(x) rep(x,5)))
lbs_camera_ref <- rep(letters[1:8],(no.replicates-5))


# design and contrast matrix for camera
des_ref <- model.matrix(~0 +as.factor(lbs_camera_ref))
colnames(des_ref) <- levels(as.factor(lbs_camera_ref))
contr_ref <- makeContrasts(e-g,levels=des_ref)



# generate splitted data with each random indices above

x <- foreach(i=1:iter) %dopar% {
    require(limma, quiet=TRUE)
    require(qusage, quiet=TRUE)
    require(gage, quiet=TRUE)
    ind <- as.numeric(rand.ind[[i]])
    expr_test <- expr.data.atlas[,ind]
    expr_ref <- expr.data.atlas[,-ind]
  
    
    gsz_ref <- mGSZm(expr.data=expr_ref,gene.sets=gene.sets.atlas,sample.labels=lbs_ref,comparisons="group5-group7",method="m1_limma",perm.number=2,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=TRUE,gsa.asymp=TRUE)
  
    mgsz_ref <- gsz_ref$comparison1$mGSZH
    mgsz_ref[,6] <- -log10(mgsz_ref[,6])
    mgsz_ref <- mgsz_ref[order(mgsz_ref[,6],mgsz_ref[,3],decreasing=T),]
    
    mgsa_ref <- gsz_ref$comparison1$mGSAH
    mgsa_ref[,6] <- -log10(mgsa_ref[,6])
    mgsa_ref <- mgsa_ref[order(mgsa_ref[,6],mgsa_ref[,3],decreasing=T),]
    
    allez_ref <- gsz_ref$comparison1$mAllezM
    allez_ref[,6] <- -log10(allez_ref[,6])
    allez_ref <- allez_ref[order(allez_ref[,6],allez_ref[,3],decreasing=T),]
    
    wks_ref <- gsz_ref$comparison1$wksM
    wks_ref[,6] <- -log10(wks_ref[,6])
    wks_ref <- wks_ref[order(wks_ref[,6],wks_ref[,3],decreasing=T),]
    
    
    # gene set analysis of ref data with camera
    camera_ref <- camera(expr_ref,gene.sets.atlas.camera,design=des_ref,contrast=contr_ref)
  
    # gene set analysis of ref data with qusage
    qusage_ref <- qsTable(qusage(expr_ref,lbs_camera_ref, "e-g", gene.sets.atlas.qusage), number=length(gene.sets.atlas.qusage))

    # gene set analysis of ref data with gage 
    endo_ref <- grep("Endothelial",colnames(expr_ref),ignore.case=TRUE)
    kera_ref <- grep("Keratinocytes",colnames(expr_ref),ignore.case=TRUE)
    less <- gage(expr_ref, gsets = gene.sets.atlas, ref = endo_ref, samp = kera_ref, compare = "unpaired",set.size=c(1,5000))$less
    greater <- gage(expr_ref, gsets = gene.sets.atlas, ref = endo_ref, samp = kera_ref, compare = "unpaired",set.size=c(1,5000))$greater
    greater <- greater[order(rownames(greater)),]
    less <-less[order(rownames(less)),]
    gage_ref <- greater
    gage_ref[,3] <- pmin(greater[,3],less[,3])
    gage_ref <- gage_ref[order(gage_ref[,3]),]
  
    # gene set analysis of ref data with romer
  
    romer.temp <- romer(y=expr_ref,index=gene.sets.atlas.romer,design=des_ref,contrast=contr_ref,nrot=2)
    intermediate <- romer.temp[,c(2,3)]
    intermediate <- apply(intermediate,1,min)
    romer.temp <- cbind(romer.temp,intermediate)
    romer_ref <- romer.temp[order(romer.temp[,5],decreasing=F),]
    
    # gene set analysis of test data with gsz based on six different permutation methods
  
    perm1.tmp <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.atlas,sample.labels=lbs_test,comparisons="group5-group7",method="m1_limma",perm.number=2,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)$comparison1$mGSZH
    # perm1.tmp[,6] <- -log10(perm1.tmp[,6])
    perm1.tmp <- perm1.tmp[order(perm1.tmp[,6],perm1.tmp[,3],decreasing=T),]
  
    perm2.tmp <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.atlas,sample.labels=lbs_test,comparisons="group5-group7",method="m2_limma",perm.number=2,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)$comparison1$mGSZH
    # perm2.tmp[,6] <- -log10(perm2.tmp[,6])
    perm2.tmp <- perm2.tmp[order(perm2.tmp[,6],perm2.tmp[,3],decreasing=T),]
    
    perm3.tmp <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.atlas,sample.labels=lbs_test,comparisons="group5-group7",method="m4_limma",perm.number=2,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)$comparison1$mGSZH
    # perm3.tmp[,6] <- -log10(perm3.tmp[,6])
    perm3.tmp <- perm3.tmp[order(perm3.tmp[,6],perm3.tmp[,3],decreasing=T),]
    
    perm4.tmp <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.atlas,sample.labels=lbs_test,comparisons="group5-group7",m="m3_limma",perm.number=2,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)$comparison1$mGSZH
    # perm4.tmp[,6] <- -log10(perm4.tmp[,6])
    perm4.tmp <- perm4.tmp[order(perm4.tmp[,6],perm4.tmp[,3],decreasing=T),]
    
    perm5.tmp <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.atlas,sample.labels=lbs_test,comparisons="group5-group7",method="m44_limma",perm.number=2,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)$comparison1$mGSZH
    # perm5.tmp[,6] <- -log10(perm5.tmp[,6])
    perm5.tmp <- perm5.tmp[order(perm5.tmp[,6],perm5.tmp[,3],decreasing=T),]
  
    perm6.tmp <- mGSZm(expr.data=expr_test,gene.sets=gene.sets.atlas,sample.labels=lbs_test,comparisons="group5-group7",method="m33_limma",perm.number=2,min.cl.sz=1,max.cl.sz=5000,gsz.asymp=FALSE,gsa.asymp=FALSE)$comparison1$mGSZH
    # perm6.tmp[,6] <- -log10(perm6.tmp[,6])
    perm6.tmp <- perm6.tmp[order(perm6.tmp[,6],perm6.tmp[,3],decreasing=T),]
   
   return(list(mgsz_ref,mgsa_ref,allez_ref,wks_ref,camera_ref,qusage_ref,gage_ref,romer_ref,perm1.tmp,perm2.tmp,perm3.tmp,perm4.tmp,perm5.tmp,perm6.tmp))
}

#######

source("generatePermCumsumDataPrimary.R")
perm.cumsumdata.primary <- generatePermCumsumDataPrimary(x,top=c(3,5,7))
source("AvgPermCumsumPlotPrimary.R")
AvgPermCumsumPlotPrimary(perm.cumsumdata.primary)







